local AddonName, GGL = ...
local GUI = GGL.GUI
local Sets = GGL.Options.Sets
local Global = {}
GGL.Options.Sets_Zone = Global

local L = GGL.Locale

-- lua
local type = type
local ipairs, pairs = ipairs, pairs

-- WoW
local GetMapInfo = C_Map.GetMapInfo

-- Libs
local AceGUI = LibStub("AceGUI-3.0")

local CONTINENT_MAP_LIST = GGL.CONTINENT_MAP_LIST
local MAP_ID_DATA = GGL.MAP_ID_DATA

-- ##################
-- local
-- ##################
local Helper


-- ##################
-- local helper
-- ##################
local function SaveDB()
    local t = Helper.Data["selectedDB"]
    for k,v in pairs(t) do
        if v and type(v) == "table" and not next(v) then
            t[k] = nil
        end
    end
    Helper:SaveDB()
end

-- ##################
-- options
-- ##################
local function EnableButton_OnValueChanged(self, event, state)
    local data = Helper.Data
    local db = data["selectedDB"]
    local continentZoneID = data["selectedTabID"]

    if state == true then
        db[continentZoneID] = nil
        SaveDB()
    elseif state == false then
        db[continentZoneID] = true
        SaveDB()
    else -- nil
        db[continentZoneID] = {}
    end

    Helper:RefreshSelectedTab()
end

local function ZoneCheck_OnValueChanged(self, event, state)
    local zoneID = self:GetUserData("zoneID")
    local data = Helper.Data
    if state then
        data["selectedDB"][ data["selectedTabID"] ][zoneID] = true
    else
        data["selectedDB"][ data["selectedTabID"] ][zoneID] = nil
    end
    SaveDB()
end

local function SetOnTabGroupSelected(content, db, tabID)
    -- top part
    local base = AceGUI:Create("SimpleGroup")
    base:SetFullWidth(true)
    base:SetLayout("Flow")
    content:AddChild(base)

    local enableCheck = AceGUI:Create("CheckBox")
    enableCheck:SetTriState(true) -- true, nil, false
    if db[tabID] == true then
        enableCheck:SetValue(false)
        enableCheck:SetLabel(L["|cffff0000Disabled|r"])
    elseif type(db[tabID]) == "table" then
        enableCheck:SetValue(nil)
        enableCheck:SetLabel(L["|cff00ff00Enabled|r"])
    else
        enableCheck:SetValue(true)
        enableCheck:SetLabel(L["|cff00ff00Enabled|r"])
    end
    enableCheck:SetCallback("OnValueChanged", EnableButton_OnValueChanged)
    base:AddChild(enableCheck)

    -- disabled skip from now
    if enableCheck:GetValue() == false then return end

    -- on nil tristate
    if enableCheck:GetValue() == nil then
        local cont = AceGUI:Create("SimpleGroup")
        cont:SetFullWidth(true)
        cont:SetLayout("Flow")
        content:AddChild(cont)

        for zoneIndex, zoneID in ipairs(CONTINENT_MAP_LIST[tabID]) do
            local info = GetMapInfo(zoneID)
            if info then
                local check = AceGUI:Create("CheckBox")
                check:SetLabel(info.name)
                check:SetValue(db[tabID][zoneID])
                check:SetCallback("OnValueChanged", ZoneCheck_OnValueChanged)
                check:SetUserData("zoneID", zoneID)
                cont:AddChild(check)
            end
        end
    end
end

local function SetOnTabGroupConfigSelected(content, db, tabID)

end

local function Options(AceGUI, container)
    Helper = Sets:GetBaseGroup(container, GGL.SET_ID_LIST.Zone)

    local tabs = {}
    for zoneIndex, zoneID in ipairs(CONTINENT_MAP_LIST.list) do
        local info = GetMapInfo(zoneID)
        if info and info.name then
            tabs[zoneIndex] = {
                value = zoneID,
                text = info.name
            }
        end
    end

    Helper:SetOnTabGroupSelected(SetOnTabGroupSelected)
    Helper:SetOnTabGroupConfigSelected(SetOnTabGroupConfigSelected)

    Helper:SetTabGroupContent(tabs, true)
end

-- Add into Options
GGL.Options:AddIntoTree(Options, L["Zone"], "sets_zone", "sets")

--[[
GGL.CONTINENT_MAP_LIST = {
	list={12,13,113,948,424,619,905,875,876},
	[12]={1,7,10,57,62,63,64,65,66,69,70,71,76,77,78,80,81,83,85,88,89,97,103,106,198,199,249,460,461,462,463,468,1339}, --Kalimdor
	[13]={14,15,17,18,21,22,23,25,26,27,32,36,37,42,47,48,49,50,51,52,56,84,87,90,94,95,110,122,201,202,203,204,205,241,210,217,224,244,245,467,425,465,469}, --Eastern Kingdoms
	[113]={114,115,116,117,118,119,120,121,123,127,170}, --Northrend
	[948]={174,194,207,276}, --The Maelstrom
	[424]={371,376,379,388,390,418,422,433,504,507,554,1530}, --Pandaria
	[619]={625,630,634,641,646,650,680,739,790}, --Broken Isles
	[905]={830,882,885}, --Argus
	[875]={862,863,864,1041,1165}, --Zandalar
	[876]={895,896,942,1039,1161,1169,1462}, --Kul Tiras
}
]]